.related a {
    margin-left: .5em;
    margin-right: .5em;
}

p.caption:not([role="heading"]) {
    text-align: center !important;
    font-weight: bold;
}

ul.parents {
    display: none;
}

.related a:hover {
    background: #7cb92f;
}
.related a[href="#"]:hover, .related a[href="index.html"]:hover {
    background: #9edb4f;
}
.related a[href="#"], .related a[href="index.html"] {
    background: #8dca3f;
}
.related a {
    padding: 0.5em;
    margin: 0;
}

.global-toc li a[href="#"]:hover, .global-toc li a[href="index.html"]:hover {
    background: #fff;
}

.global-toc li a[href="#"], .global-toc li a[href="index.html"] {
    background: #fafafa;
}

.global-toc li a {
    display: block;
}

.global-toc li a:hover {
    background: #f5f5f5;
}

div.body h1 {
    background: #beebbe;
}

div.body h2 {
    background: #c8e3c8;
}

div.body h3, div.body h4, div.body h5, div.body h6 {
    background: #d9e3d8;
}

div.inheritance_box {
    overflow: auto;
}

tt.docutils.literal {
    background: transparent;
    font-size: 0.9em;
}

dl.class {
    border-top: 1px solid #5a970d;
    padding-top: 15px;
}

dl.class.rm_expanded, dl.class.rm_collapsed {
    border-top: none;
    padding: 0;
    margin-top: 0;
    margin-bottom: 0;
    /*padding-left: 1.5em;*/
}

dl.class.rm_expanded > dt:before {
    content: "[-]";
    cursor: pointer;
    color: #005B81;
}

dl.class.rm_collapsed dt:before {
    content: "[+]";
    cursor: pointer;
    color: #005B81;
}

dl.class.rm_expanded dd {
    display: block;
}

dl.class.rm_collapsed dd {
    display: none;
}

a.anchor-link {
    visibility: hidden;
}

div.input_prompt {
    visibility: hidden;
    height: 0;
}

div.output_prompt {
    visibility: hidden;
    height: 0;
}

.output_subarea pre {
	background-color: transparent;
	border-style: none;
	margin-left: 1em;
}

table.dataframe {
    font-family: Arial, sans-serif;
    font-size: 13px;
    line-height: 20px;
	margin-left: 2em;
	margin-bottom: 2em;
}

table.dataframe th, td {
    padding: 4px;
    text-align: left;
}

.rendered_html td {
    border-style: solid;
    border-width: 1px;
    word-wrap: break-word;
}

.rendered_html code {
	font-size: 0.9em !important;
}

.bk-plot ul {
    margin-bottom: 2px !important;
}

.bk-plot-wrapper td{
    border-style: none !important;
}

/*
Sphinx-Gallery has compatible CSS to fix default sphinx themes
Tested for Sphinx 1.3.1 for all themes: default, alabaster, sphinxdoc,
scrolls, agogo, traditional, nature, haiku, pyramid
Tested for Read the Docs theme 0.1.7 */
.sphx-glr-thumbcontainer {
  background: #fff;
  border: solid #fff 1px;
  -moz-border-radius: 5px;
  -webkit-border-radius: 5px;
  border-radius: 5px;
  box-shadow: none;
  float: left;
  margin: 0px;
  min-height: 210px;
  padding-top: 0px;
  position: relative;
}
.sphx-glr-thumbcontainer:hover {
  border: solid #b4ddfc 1px;
  box-shadow: 0 0 15px rgba(142, 176, 202, 0.5);
}
.sphx-glr-thumbcontainer a.internal {
  bottom: 0;
  display: block;
  left: 0;
  padding: 165px 0 0;
  position: absolute;
  right: 0;
  top: 0;
}
/* Next one is to avoid Sphinx traditional theme to cover all the
thumbnail with its default link Background color */
.sphx-glr-thumbcontainer a.internal:hover {
  background-color: transparent;
}

.sphx-glr-thumbcontainer p {
  margin: 0 0 .1em 0;
}
.sphx-glr-thumbcontainer .figure {
  margin: 0px;
  width: 160px;
  padding: 0.1em;
}
.sphx-glr-thumbcontainer img {
  display: inline;
  max-height: 160px;
  width: 160px;
}
.sphx-glr-thumbcontainer[tooltip]:hover:after {
  background: rgba(0, 0, 0, 0.8);
  -webkit-border-radius: 5px;
  -moz-border-radius: 5px;
  border-radius: 5px;
  color: #fff;
  content: attr(tooltip);
  left: 95%;
  padding: 5px 15px;
  position: absolute;
  z-index: 98;
  width: 165px;
  bottom: 52%;
}
.sphx-glr-thumbcontainer[tooltip]:hover:before {
  border: solid;
  border-color: #333 transparent;
  border-width: 18px 0 0 20px;
  bottom: 58%;
  content: '';
  left: 85%;
  position: absolute;
  z-index: 99;
}
.sphx-glr-thumbcontainer a.headerlink {
  color: transparent;
}

.highlight-pytb pre {
  background-color: #ffe4e4;
  border: 1px solid #f66;
  margin-top: 10px;
  padding: 7px;
}

.sphx-glr-script-out {
  color: #888;
  margin: 0;
}
.sphx-glr-script-out .highlight {
  background-color: transparent;
  margin-left: 2.5em;
  margin-top: -1.4em;
}
.sphx-glr-script-out .highlight pre {
  background-color: #fafae2;
  border: 0;
  max-height: 30em;
  overflow: auto;
  padding-left: 1ex;
  margin: 0px;
  word-break: break-word;
}
.sphx-glr-script-out + p {
  margin-top: 1.8em;
}
blockquote.sphx-glr-script-out {
  margin-left: 0pt;
}

div.sphx-glr-footer {
    text-align: center;
}

div.sphx-glr-download {
  display: inline-block;
  margin: 1em auto 1ex 2ex;
  vertical-align: middle;
}

div.sphx-glr-download a {
  background-color: #ffc;
  background-image: linear-gradient(to bottom, #FFC, #d5d57e);
  border-radius: 4px;
  border: 1px solid #c2c22d;
  color: #000;
  display: inline-block;
  /* Not valid in old browser, hence we keep the line above to override */
  display: table-caption;
  font-weight: bold;
  padding: 1ex;
  text-align: center;
}

/* The last child of a download button is the file name */
div.sphx-glr-download a span:last-child {
    font-size: smaller;
}

@media (min-width: 20em) {
    div.sphx-glr-download a {
	min-width: 10em;
    }
}

@media (min-width: 30em) {
    div.sphx-glr-download a {
	min-width: 13em;
    }
}

@media (min-width: 40em) {
    div.sphx-glr-download a {
	min-width: 16em;
    }
}



div.sphx-glr-download code.download {
  display: inline-block;
  white-space: normal;
  word-break: normal;
  overflow-wrap: break-word;
  /* border and background are given by the enclosing 'a' */
  border: none;
  background: none;
}

div.sphx-glr-download a:hover {
  box-shadow: inset 0 1px 0 rgba(255,255,255,.1), 0 1px 5px rgba(0,0,0,.25);
  text-decoration: none;
  background-image: none;
  background-color: #d5d57e;
}

ul.sphx-glr-horizontal {
  list-style: none;
  padding: 0;
}
ul.sphx-glr-horizontal li {
  display: inline;
}
ul.sphx-glr-horizontal img {
  height: auto !important;
}

p.sphx-glr-signature a.reference.external {
  -moz-border-radius: 5px;
  -webkit-border-radius: 5px;
  border-radius: 5px;
  padding: 3px;
  font-size: 75%;
  text-align: right;
  margin-left: auto;
  display: table;
}

ul.tab {
    list-style-type:none;
    margin: 0;
    padding:0;
    border-radius: 5px;
    overflow: hidden;
}
ul.tab li {
    list-style-type: none !important;
    float: left;
    padding: 0;
}

ul.tab li label {
  background: white;
  padding: 6px;
  border: 1px solid #ccc;
  display: inline-block;
}
ul.tab li input[type="radio"] {
    opacity: 0;
    width:1px;
    height:1px;
}
ul.tab li input[type="radio"]:checked ~ label {
    background: #902040;
    color: white;
}

/* CSS for pandas tables (dataframes) */

.rendered_html table {
    margin-left: auto;
    margin-right: auto;
    border: none;
    border-collapse: collapse;
    border-spacing: 0;
    color: black;
    font-size: 12px;
    table-layout: fixed;
}
.rendered_html tr, .rendered_html th, .rendered_html td {
    text-align: right;
    vertical-align: middle;
    padding: 0.5em 0.5em;
    line-height: normal;
    white-space: normal;
    max-width: none;
    border: none;
}
tbody {
    display: table-row-group;
    vertical-align: middle;
    border-color: inherit;
}
.rendered_html tbody tr:nth-child(odd) {
    background: #f5f5f5;
}

.rendered_html thead {
    border-bottom: 1px solid black;
    vertical-align: bottom;
}

dl.dl-horizontal {
	padding-left: 50px;
}

.alert {
  padding: 0.75rem 1.25rem;
  margin-bottom: 1rem;
  border: 1px solid transparent;
  border-radius: 0.25rem;
}

.alert-heading {
  color: inherit;
}

.alert-link {
  font-weight: bold;
}

.alert-dismissible .close {
  position: relative;
  top: -0.75rem;
  right: -1.25rem;
  padding: 0.75rem 1.25rem;
  color: inherit;
}

.alert-success {
  background-color: #dff0d8;
  border-color: #d0e9c6;
  color: #3c763d;
}

.alert-success hr {
  border-top-color: #c1e2b3;
}

.alert-success .alert-link {
  color: #2b542c;
}

.alert-info {
  background-color: #d9edf7;
  border-color: #bcdff1;
  color: #31708f;
}

.alert-info hr {
  border-top-color: #a6d5ec;
}

.alert-info .alert-link {
  color: #245269;
}

.alert-warning {
  background-color: #fcf8e3;
  border-color: #faf2cc;
  color: #8a6d3b;
}

.alert-warning hr {
  border-top-color: #f7ecb5;
}

.alert-warning .alert-link {
  color: #66512c;
}

.alert-danger {
  background-color: #f2dede;
  border-color: #ebcccc;
  color: #a94442;
}

.alert-danger hr {
  border-top-color: #e4b9b9;
}

.alert-danger .alert-link {
  color: #843534;
}

/* Avoid wrapping in HoloViews extension logo block */
div.logo-block {
  display: inline-block;
}

div.bk-root {
  min-height: 50px
}

#scroller-right {
  position: fixed;
  top: 80%;
  right: 1%;
  max-width: 10%;
  background: #f2f2f2;
  padding: 0.8%;
  transform: translate(0%, -50%);
  border: 1px solid black;
  font-size: smaller;
}

.bk.accordion-header {
  color: black;
}

.bk.card-header {
  color: black;
}

.cell_output {
  padding-left: 0;
}

.bk.bk-root {
  display: none;
}

.output {
  overflow-x: auto;
}

/* Adapted from the sphinx book theme */
main.bd-content a.headerlink {
  opacity:0;
  margin-left:.2em
 }

 main.bd-content a.headerlink:hover {
  background-color:transparent;
  color:rgba(var(--pst-color-headerlink-hover), 1);
  opacity:1 !important
 }
 
 main.bd-content h1:hover a.headerlink,
 main.bd-content h2:hover a.headerlink,
 main.bd-content h3:hover a.headerlink,
 main.bd-content h4:hover a.headerlink,
 main.bd-content h5:hover a.headerlink {
  opacity:.5
 }
 /* End of copy  */

 /* Color of the  paragraph symbol of the header link */
 :root {
  --pst-color-headerlink: 170,170,170;
  --pst-color-headerlink-hover: 170,170,170;
}

/* White icons in the nav bar instead of the default colors set by pydata sphinx theme */
#navbar-icon-links i.fa-github-square:before {
  color: white;
}

#navbar-icon-links i.fa-discourse:before {
  color: white;
}

#navbar-icon-links i.fa-twitter-square:before {
  color: white;
}

/* Binder links displayed in the right sidebar when the page is built from a notebook */
.bd-toc #binder-link {
  display: inline-block;
  font-size: 0.9rem;
  padding-left: 1.5rem;
  padding-top: 1rem;
  padding-bottom: 1rem;
}

/* Hide cell toggle */
details.toggle-details {
  display: none;
}
